<?php

namespace Drupal\group_flex\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupTypeInterface;

/**
 * Base class for Group visibility plugins.
 */
abstract class GroupVisibilityBase extends PluginBase implements GroupVisibilityInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  public $entityTypeManager;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = \Drupal::service('entity_type.manager');
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->pluginDefinition['weight'];
  }

  /**
   * {@inheritdoc}
   */
  public function getGroupPermissions(GroupInterface $group) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getDisallowedGroupPermissions(GroupInterface $group) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function saveMappedPerm(array $mappedPerm, GroupTypeInterface $groupType) {
    foreach ($mappedPerm as $role_name => $permissions) {
      $group_role_storage = $this->entityTypeManager->getStorage('group_role');
      /** @var \Drupal\group\Entity\GroupRoleInterface $group_role */
      $group_role = $group_role_storage->load($role_name);

      if ($group_role && !empty($permissions)) {
        foreach ($permissions as $perm => $value) {
          if ($value === TRUE) {
            $group_role->grantPermission($perm);
            continue;
          }
          $group_role->revokePermission($perm);
        }
        $group_role->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getGroupLabel(GroupTypeInterface $groupType) {
    return $this->getValueDescription($groupType);
  }

  /**
   * {@inheritdoc}
   */
  public function getValueDescription(GroupTypeInterface $groupType) {
    return '';
  }

}
